from hopper_api.app import App
from hopper_api.crypto import *
from hopper_api.notification import Notification, Action
import requests
import json

HopperProd = ["https://api.hoppercloud.net/v1", "https://app.hoppercloud.net/subscribe"]
HopperDev = ["https://api-dev.hoppercloud.net/v1", "https://dev.hoppercloud.net/subscribe"]

class HopperApi:
    """API Connection to all Hopper related tasks"""
    
    def __init__(self, hopperEnv):
        self.baseUrl = hopperEnv[0]
        self.subscribeUrl = hopperEnv[1]


    def deserialize_app(self, serialized: str) -> App:
        """Convert a JSON-Serialized app into an actual App

           Returns: The deserialized App's object
        """

        obj = json.loads(serialized)
        return App(self, obj["id"], decode_private_key_base64(obj["key"]))

    
    def check_connectivity(self) -> bool:
        """Checks whether Hopper can be reached with the current parameters

           Returns: Whether the check was successful
        """

        try:
            res = requests.get(self.baseUrl)
            if (res.json()["type"]):
                print("You are using a DEV instance of Hopper! This is not intended for production!")
        except ConnectionError as e:
            print(json.dumps(e))
            return False
        return True


    def create_app(self, name: str, baseUrl: str, imageUrl: str, manageUrl: str, contactEmail: str) -> App:
        """Creates an App and registers it with hopper

           Returns: The registed App's object
        """

        (pub, priv) = generate_keys()
        res = requests.post(self.baseUrl + '/app', json={
            "name": name,
            "baseUrl": baseUrl,
            "imageUrl": imageUrl,
            "manageUrl": manageUrl,
            "contactEmail": contactEmail,
            "cert": encode_key_base64(pub)            
        })

        json = res.json()
        if res.status_code != 200:
            if "reason" in json:
                raise ConnectionError(json['reason'])
            raise ConnectionError(json)
        
        return App(self, json['id'], priv)


    def post_notification(self, subscriptionId: str, notification: Notification) -> str:
        """Posts a notification on the given subscription

           Returns: The Id of the posted notification
        """

        data = notification.data
        
        res = requests.post(self.baseUrl + '/notification', json={
            'subscriptionId': subscriptionId,
            'notification': notification.data
        })

        json_res = res.json()
        if res.status_code != 200:
            if "reason" in json_res:
                raise ConnectionError(json_res['reason'])
            raise ConnectionError(json_res)
 
        return json_res['id']

    
    def update_notification(self, notificationId: str, heading: str=None, timestamp: str=None, imageUrl: str=None, isDone: str=None, isSilent: str=None, content: str=None, actions: [Action]=None):
        """Updates an already posted notification"""
        
        data = {}
        if heading is not None:
            data['heading'] = heading

        if timestamp is not None:
            data['timestamp'] = timestamp

        if imageUrl is not None:
            data['imageUrl'] = imageUrl

        if isDone is not None:
            data['isDone'] = isDone

        if isSilent is not None:
            data['isSilent'] = isSilent

        if content is not None:
            data['content'] =  content
 
        if actions is not None:
            data['actions'] = actions


        res = requests.put(self.baseUrl + '/notification', json={
            "id": notificationId,
            "notification": data
        })

        json_res = res.json()
        if res.status_code != 200:
            if "reason" in json_res:
                raise ConnectionError(json_res['reason'])
            raise ConnectionError(json_res)

            
    def delete_notification(self, notificationId: str):
        """Deletes an already posted notification"""
        res = requests.delete(self.baseUrl + '/notification?id=' + notificationId)

        json_res = res.json()
        if res.status_code != 200:
            if "reason" in json_res:
                raise ConnectionError(json_res['reason'])
            raise ConnectionError(json_res)
 



from datetime import datetime
from datetime import timezone

class Notification:
    @staticmethod
    def default(heading: str, content: str) -> 'Notification':
        """Creates a default notification with the given paramenters
        
           Returns: The created Notification
        """

        return Notification("default", heading, content, int(datetime.now().timestamp()*1000), False, False, [])

 
    def __init__(self, notificationType, heading, content, timestamp, isDone, isSilent, actions):
        self.data = {
            'type': notificationType,
            'heading': heading,
            'content': content,
            'timestamp': timestamp,
            'isDone': isDone,
            'isSilent': isSilent,
            'actions': []
        }
        print(timestamp)


    def is_done(self, val: bool) -> 'Notification':
        """Sets isDone

           Returns: The updated notification
        """

        self.data['isDone'] = val
        return self
        

    def is_silent(self, val: bool) -> 'Notification':
        """Sets isSilent

           Returns: The updated notification
        """

        self.data['isSilent'] = val
        return self
    

    def timestamp(self, val: int) -> 'Notification':
        """Sets the notification's timestamp

           Returns: The updated notification
        """

        self.data['timestamp'] = val
        return self

   
    def action(self, action_obj: 'Action') -> 'Notification':
        """Adds the action to the notification

           Returns: The updated notification
        """

        self.data['actions'].append(action_obj.data)
        return self


    def actions(self, action_ary: ['Action']) -> 'Notification':
        """Override all previous added actions with the given array

           Returns: The updated notification
        """

        self.data['actions'] = [x.data for x in action_ary]
        return self



class Action:
    @staticmethod
    def submit(text: str, url: str) -> 'Action':
        """Creates a an action of type submit
        
           Returns: The created Action
        """

        return Action('submit', text, url, False)

    
    @staticmethod
    def text(text: str, url: str) -> 'Action':
        """Creates a an action of type text
        
           Returns: The created Action
        """

        return Action('text', text, url, False)


    @staticmethod
    def redirect(text: str, url: str) -> 'Action':
        """Creates a an action of type redirect
        
           Returns: The created Action
        """

        return Action('text', text, url, False)


    def __init__(self, actionType, text, url, markAsDone):
        self.data = {
            'type': actionType,
            'text': text,
            'url': url,
            'markAsDone': markAsDone
        }

    
    def mark_as_done(self, val: bool) -> 'Action':
        """Sets whether triggering the action marks the notification as done

           Returns: The updated notification
        """

        self.data['markAsDone'] = val
        return self





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hopper_api",
    version="0.3.2",
    author="The hopper team",
    author_email="info@hoppercloud.net",
    description="Hopper's app API",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/hopper-team/hopper-api",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pyjwt',
        'pyjwt[crypto]'
    ],
    python_requires='>=3.6',
)



from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
import base64
import jwt

def generate_keys():
    privateKey = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend()
    )
    return (privateKey.public_key().public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.PKCS1
    ), privateKey.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption()
    ))

def sign(obj, privateKey):
    return jwt.encode(obj, privateKey, 'RS256').decode('utf-8')    

def encode_key_base64(key):
    return base64.b64encode(key).decode('utf-8')

def decode_private_key_base64(b64):
    return base64.b64decode(b64.encode('utf-8'))




import json
import base64
from hopper_api.crypto import *
import requests

class App:
    def __init__(self, api, id, privateKey):
        self.api = api
        self.id = id
        self.privateKey = privateKey


    def __send_update(self, data):
        res = requests.put(self.api.baseUrl + '/app', json={
            "id": self.id, 
            "content": sign(data, self.privateKey)
        })
        json_res = res.json()
        if res.status_code != 200:
            if "reason" in json_res:
                raise ConnectionError(json_res['reason'])
            raise ConnectionError(json_res)

    
    def update(self, name: str = None, imageUrl: str = None, manageUrl: str = None, contactEmail: str = None):
        """Updates the app's metadata at Hopper"""
        
        data = {}
        if name is not None:
            data['name'] = name

        if imageUrl is not None:
            data['imageUrl'] = imageUrl

        if manageUrl is not None:
            data['manageUrl'] = manageUrl

        if contactEmail is not None:
            data['contactEmail'] = contactEmail

        self.__send_update(data)

    
    def create_subscribe_request(self, callback: str, accountName: str=None) -> str:
        """Create a subscription request

           Returns: A url to which the user has to be forwarded
        """
        
        subReq = {
            "callback": callback,
            "requestedInfos": []
        }
        if accountName is not None:
            subReq['accountName'] = accountName

        encSubReq = sign(subReq, self.privateKey)

        return self.api.subscribeUrl + "?id=" + self.id + "&content=" + encSubReq


    def generate_new_keys(self):
        """Generates new keys for this App and update them at Hopper"""

        (pub, priv) = generate_keys()
        self.__send_update({
            "cert": encode_key_base64(pub)
        });
        self.privateKey = priv
        return True


    def serialize(self) -> str:
        """Serialize this App into a JSON-String

           Returns: The serialized app as string
        """

        return json.dumps({
            "id": self.id,
            "key": encode_key_base64(self.privateKey)
        })
